<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Shipping Restriction Restricted Products by Amasty
 */

namespace Amasty\ShiprestrictionRestrictedProducts\Plugin\SalesRule\Model\Rule\Condition\Product\Found;

use Amasty\Shiprestriction\Model\ProductRegistry;
use Amasty\Shiprestriction\Model\Rule;
use Magento\Framework\Model\AbstractModel;
use Magento\Quote\Model\Quote\Item;
use Magento\Rule\Model\Condition\Combine;
use Magento\SalesRule\Model\Rule\Condition\Product\Found;

class SaveRestrictedProducts
{
    /**
     * @var Combine
     */
    private $combine;

    /**
     * @var ProductRegistry
     */
    private $productRegistry;

    public function __construct(
        Combine $combine,
        ProductRegistry $productRegistry
    ) {
        $this->combine = $combine;
        $this->productRegistry = $productRegistry;
    }

    public function afterValidate(Found $subject, bool $result, AbstractModel $model): bool
    {
        $rule = $subject->getRule();

        if (!$this->needToStoreProducts($rule)) {
            return $result;
        }

        // result of \Magento\SalesRule\Model\Rule\Condition\Product::validate() method
        // for \Amasty\Shiprestriction\Model\Rule rules
        $productData = $this->productRegistry->getProductValidationData();
        $this->combine->setConditions($subject->getConditions());

        // To save restricted items we need to validate all items.
        // Origin function breaks when find first valid item.
        foreach ($model->getAllItems() as $item) {
            if (($item instanceof Item)
                && (!array_key_exists($item->getSku(), $productData))
            ) {
                $this->combine->validate($item);
            }
        }

        if (!$subject->getValue()) {
            $this->changeProductData();
        }

        return $result;
    }

    /**
     * @param \Magento\Rule\Model\AbstractModel $rule
     * @return bool
     */
    private function needToStoreProducts($rule): bool
    {
        if (!$rule instanceof Rule) {
            return false;
        }

        $message = $rule->getCustomRestrictionMessage();

        // Saving Restricted products can decrease performance.
        // We should do it only if we want to show products in restriction message.
        return str_contains($message, '{restricted-products}') || str_contains($message, '{last-restricted-product}');
    }

    private function changeProductData()
    {
        $productData = $this->productRegistry->getProductValidationData();
        foreach ($productData as &$product) {
            $product['validation_result'] = !$product['validation_result'];
        }

        $this->productRegistry->addProductValidationData($productData);
    }
}
